<?php

namespace Drupal\monarch_migration_tools\Plugin\migrate\source;

use Drupal\Core\Config\FileStorage;
use Drupal\Core\File\FileSystemInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate\Plugin\migrate\source\SourcePluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Allows source data to be defined in the configuration of the source plugin.
 *
 * Technically all keys are optional, but it works better if you specify at
 * least a module or directory (preferably both), otherwise it will attempt to
 * load files from the root of the drupal install.
 *
 * Available configuration keys:
 * - module: The module to find the directory in. (optional)
 * - directory: The directory to get the YAML files from. (default: config)
 * - collection: The configuration collection. (optional)
 * - prefix: The file prefix to search with. (optional)
 *
 * @MigrateSource(
 *   id = "yaml_files",
 *   source_module = "monarch_migration_tools"
 * )
 */
class YamlFiles extends SourcePluginBase implements ContainerFactoryPluginInterface {

  /**
   * Data obtained from the source plugin configuration.
   *
   * @var array[]
   *   Array of data rows, each one an array of values keyed by field names.
   */
  protected $dataRows = [];

  /**
   * Description of the unique ID fields for this source.
   *
   * @var array[]
   *   Each array member is keyed by a field name, with a value that is an
   *   array with a single member with key 'type' and value a column type such
   *   as 'integer'.
   */
  protected $ids = [];

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration, ModuleHandlerInterface $module_handler, FileSystemInterface $file_system) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);

    $module_name = $configuration['module'] ?? '';
    $collection = $configuration['collection'] ?? '';
    $dir = $configuration['directory'] ?? 'config';
    $prefix = $configuration['prefix'] ?? '';
    $storage = NULL;
    $data = NULL;
    $this->dataRows = [];

    if (empty($module_name)) {
      $full_path = $file_system->realpath($dir);
    }
    else {
      $module = $module_handler->getModule($module_name);

      if (empty($module)) {
        $full_path = $file_system->realpath($dir);
      }
      else {
        $full_path = $file_system->realpath($module->getPath() . '/' . $dir);
      }
    }

    if (empty($full_path)) {
      return;
    }

    $storage = new FileStorage(
      $full_path,
      $collection
    );

    $data = $storage->readMultiple($storage->listAll($prefix));

    foreach ($data as $name => $config_data) {
      $this->dataRows[] = [
        'name' => $name,
        'data' => $config_data,
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('module_handler'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return [
      'name' => [
        'type' => 'string',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    return [
      'name' => $this->t('Configuration Name'),
      'data' => $this->t('Configuration Data'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function initializeIterator() {
    return new \ArrayIterator($this->dataRows);
  }

  /**
   * {@inheritdoc}
   */
  public function __toString() {
    return 'YAML Files';
  }

  /**
   * {@inheritdoc}
   */
  public function count($refresh = FALSE) {
    return count($this->dataRows);
  }

}
